@@ -17,13 +17,12 @@
 * unsutable for anything else.
 ******************************************************************************/
 
-#include <unistd.h>
+#include <strings.h>
 #include <string.h>
 #include <time.h>
 #include <stdio.h>
 #include <fcntl.h>
 #include <sys/stat.h>
-#include <strings.h>
 
 #include "bk.h"
 #include "bkInternal.h"
@@ -37,6 +36,7 @@
 #include "bkCache.h"
 #include "bkRead7x.h"
 #include "bkLink.h"
+#include "bkIoWrappers.h"
 
 /******************************************************************************
 * bk_write_image()
@@ -48,21 +48,20 @@ int bk_write_image(const char* newImagePathAndName, VolInfo* volInfo,
                    void(*progressFunction)(VolInfo*, double))
 {
     int rc;
-    struct stat statStruct;
     DirToWrite newTree;
-    off_t svdOffset;
-    off_t pRealRootDrOffset;
+    bk_off_t svdOffset;
+    bk_off_t pRealRootDrOffset;
     int pRootDirSize;
-    off_t sRealRootDrOffset;
+    bk_off_t sRealRootDrOffset;
     int sRootDirSize;
-    off_t lPathTable9660Loc;
-    off_t mPathTable9660Loc;
+    bk_off_t lPathTable9660Loc;
+    bk_off_t mPathTable9660Loc;
     int pathTable9660Size;
-    off_t lPathTableJolietLoc;
-    off_t mPathTableJolietLoc;
+    bk_off_t lPathTableJolietLoc;
+    bk_off_t mPathTableJolietLoc;
     int pathTableJolietSize;
-    off_t bootCatalogSectorNumberOffset;
-    off_t currPos;
+    bk_off_t bootCatalogSectorNumberOffset;
+    bk_off_t currPos;
     
     volInfo->writeProgressFunction = progressFunction;
     volInfo->stopOperation = false;
@@ -70,13 +69,14 @@ int bk_write_image(const char* newImagePathAndName, VolInfo* volInfo,
     volInfo->estimatedIsoSize = bk_estimate_iso_size(volInfo, filenameTypes);
     progressFunction(volInfo, 0);
     
-    rc = stat(newImagePathAndName, &statStruct);
+    BkStatStruct statStruct;
+    rc = bkStat(newImagePathAndName, &statStruct);
     if(rc == 0 && statStruct.st_ino == volInfo->imageForReadingInode)
         return BKERROR_SAVE_OVERWRITE;
     
     /* because mangleDir works on dir's children i need to 
     * copy the root manually */
-    bzero(&newTree, sizeof(DirToWrite));
+    memset(&newTree, 0, sizeof(DirToWrite));
     newTree.base.posixFileMode = volInfo->dirTree.base.posixFileMode;
     
     printf("mangling\n");fflush(NULL);
@@ -87,10 +87,10 @@ int bk_write_image(const char* newImagePathAndName, VolInfo* volInfo,
         freeDirToWriteContents(&newTree);
         return rc;
     }
-    //~ printDirToWrite(&newTree, 0, filenameTypes);
+    
     printf("opening '%s' for writing\n", newImagePathAndName);fflush(NULL);
     volInfo->imageForWriting = open(newImagePathAndName, 
-                                    O_WRONLY | O_CREAT | O_TRUNC, 
+                                    O_RDWR | O_CREAT | O_TRUNC, 
                                     S_IRUSR | S_IWUSR);
     if(volInfo->imageForWriting == -1)
     {
@@ -104,7 +104,7 @@ int bk_write_image(const char* newImagePathAndName, VolInfo* volInfo,
     if(rc <= 0)
     {
         freeDirToWriteContents(&newTree);
-        close(volInfo->imageForWriting);
+        bkClose(volInfo->imageForWriting);
         unlink(newImagePathAndName);
         return rc;
     }
@@ -119,7 +119,7 @@ int bk_write_image(const char* newImagePathAndName, VolInfo* volInfo,
         if(rc <= 0)
         {
             freeDirToWriteContents(&newTree);
-            close(volInfo->imageForWriting);
+            bkClose(volInfo->imageForWriting);
             unlink(newImagePathAndName);
             return rc;
         }
@@ -138,7 +138,7 @@ int bk_write_image(const char* newImagePathAndName, VolInfo* volInfo,
     if(rc <= 0)
     {
         freeDirToWriteContents(&newTree);
-        close(volInfo->imageForWriting);
+        bkClose(volInfo->imageForWriting);
         unlink(newImagePathAndName);
         return rc;
     }
@@ -152,7 +152,7 @@ int bk_write_image(const char* newImagePathAndName, VolInfo* volInfo,
         if(rc <= 0)
         {
             freeDirToWriteContents(&newTree);
-            close(volInfo->imageForWriting);
+            bkClose(volInfo->imageForWriting);
             unlink(newImagePathAndName);
             return rc;
         }
@@ -163,7 +163,7 @@ int bk_write_image(const char* newImagePathAndName, VolInfo* volInfo,
         if(rc <= 0)
         {
             freeDirToWriteContents(&newTree);
-            close(volInfo->imageForWriting);
+            bkClose(volInfo->imageForWriting);
             unlink(newImagePathAndName);
             return rc;
         }
@@ -182,16 +182,16 @@ int bk_write_image(const char* newImagePathAndName, VolInfo* volInfo,
         if(volInfo->bootRecordIsOnImage)
         {
             srcFile = volInfo->imageForReading;
-            lseek(volInfo->imageForReading, volInfo->bootRecordOffset, SEEK_SET);
+            bkSeekSet(volInfo->imageForReading, volInfo->bootRecordOffset, SEEK_SET);
             srcFileOpened = false;
         }
         else
         {
-            srcFile = open(volInfo->bootRecordPathAndName, O_RDONLY);
+            srcFile = open(volInfo->bootRecordPathAndName, O_RDONLY, 0);
             if(srcFile == -1)
             {
                 freeDirToWriteContents(&newTree);
-                close(volInfo->imageForWriting);
+                bkClose(volInfo->imageForWriting);
                 unlink(newImagePathAndName);
                 return BKERROR_OPEN_READ_FAILED;
             }
@@ -207,8 +207,8 @@ int bk_write_image(const char* newImagePathAndName, VolInfo* volInfo,
         {
             freeDirToWriteContents(&newTree);
             if(srcFileOpened)
-                close(srcFile);
-            close(volInfo->imageForWriting);
+                bkClose(srcFile);
+            bkClose(volInfo->imageForWriting);
             unlink(newImagePathAndName);
             return rc;
         }
@@ -220,8 +220,8 @@ int bk_write_image(const char* newImagePathAndName, VolInfo* volInfo,
         {
             freeDirToWriteContents(&newTree);
             if(srcFileOpened)
-                close(srcFile);
-            close(volInfo->imageForWriting);
+                bkClose(srcFile);
+            bkClose(volInfo->imageForWriting);
             unlink(newImagePathAndName);
             return rc;
         }
@@ -235,20 +235,19 @@ int bk_write_image(const char* newImagePathAndName, VolInfo* volInfo,
         {
             freeDirToWriteContents(&newTree);
             if(srcFileOpened)
-                close(srcFile);
-            close(volInfo->imageForWriting);
+                bkClose(srcFile);
+            bkClose(volInfo->imageForWriting);
             unlink(newImagePathAndName);
             return rc;
         }
         
         if(srcFileOpened)
-            close(srcFile);
+            bkClose(srcFile);
     }
     /* END MAYBE write boot record file now */
     
     printf("sorting 9660\n");
     sortDir(&newTree, FNTYPE_9660);
-    //printDirToWrite(&newTree, 0, FNTYPE_9660 | FNTYPE_JOLIET);
     
     pRealRootDrOffset = wcSeekTell(volInfo);
     
@@ -259,7 +258,7 @@ int bk_write_image(const char* newImagePathAndName, VolInfo* volInfo,
     if(rc <= 0)
     {
         freeDirToWriteContents(&newTree);
-        close(volInfo->imageForWriting);
+        bkClose(volInfo->imageForWriting);
         unlink(newImagePathAndName);
         return rc;
     }
@@ -280,7 +279,7 @@ int bk_write_image(const char* newImagePathAndName, VolInfo* volInfo,
         if(rc <= 0)
         {
             freeDirToWriteContents(&newTree);
-            close(volInfo->imageForWriting);
+            bkClose(volInfo->imageForWriting);
             unlink(newImagePathAndName);
             return rc;
         }
@@ -295,7 +294,7 @@ int bk_write_image(const char* newImagePathAndName, VolInfo* volInfo,
     if(rc <= 0)
     {
         freeDirToWriteContents(&newTree);
-        close(volInfo->imageForWriting);
+        bkClose(volInfo->imageForWriting);
         unlink(newImagePathAndName);
         return rc;
     }
@@ -306,7 +305,7 @@ int bk_write_image(const char* newImagePathAndName, VolInfo* volInfo,
     if(rc <= 0)
     {
         freeDirToWriteContents(&newTree);
-        close(volInfo->imageForWriting);
+        bkClose(volInfo->imageForWriting);
         unlink(newImagePathAndName);
         return rc;
     }
@@ -319,7 +318,7 @@ int bk_write_image(const char* newImagePathAndName, VolInfo* volInfo,
         if(rc <= 0)
         {
             freeDirToWriteContents(&newTree);
-            close(volInfo->imageForWriting);
+            bkClose(volInfo->imageForWriting);
             unlink(newImagePathAndName);
             return rc;
         }
@@ -330,7 +329,7 @@ int bk_write_image(const char* newImagePathAndName, VolInfo* volInfo,
         if(rc <= 0)
         {
             freeDirToWriteContents(&newTree);
-            close(volInfo->imageForWriting);
+            bkClose(volInfo->imageForWriting);
             unlink(newImagePathAndName);
             return rc;
         }
@@ -343,7 +342,7 @@ int bk_write_image(const char* newImagePathAndName, VolInfo* volInfo,
     if(rc <= 0)
     {
         freeDirToWriteContents(&newTree);
-        close(volInfo->imageForWriting);
+        bkClose(volInfo->imageForWriting);
         unlink(newImagePathAndName);
         return rc;
     }
@@ -355,7 +354,7 @@ int bk_write_image(const char* newImagePathAndName, VolInfo* volInfo,
         if(rc <= 0)
         {
             freeDirToWriteContents(&newTree);
-            close(volInfo->imageForWriting);
+            bkClose(volInfo->imageForWriting);
             unlink(newImagePathAndName);
             return rc;
         }
@@ -370,7 +369,7 @@ int bk_write_image(const char* newImagePathAndName, VolInfo* volInfo,
     if(rc <= 0)
     {
         freeDirToWriteContents(&newTree);
-        close(volInfo->imageForWriting);
+        bkClose(volInfo->imageForWriting);
         unlink(newImagePathAndName);
         return rc;
     }
@@ -386,7 +385,7 @@ int bk_write_image(const char* newImagePathAndName, VolInfo* volInfo,
         if(rc <= 0)
         {
             freeDirToWriteContents(&newTree);
-            close(volInfo->imageForWriting);
+            bkClose(volInfo->imageForWriting);
             unlink(newImagePathAndName);
             return rc;
         }
@@ -394,7 +393,7 @@ int bk_write_image(const char* newImagePathAndName, VolInfo* volInfo,
     
     printf("freeing memory\n");fflush(NULL);
     freeDirToWriteContents(&newTree);
-    close(volInfo->imageForWriting);
+    bkClose(volInfo->imageForWriting);
     
     return 1;
 }
@@ -405,25 +404,28 @@ int bk_write_image(const char* newImagePathAndName, VolInfo* volInfo,
 * */
 int bootInfoTableChecksum(int oldImage, FileToWrite* file, unsigned* checksum)
 {
-    int rc;
+    int numTrailingBytes; /* to make sure the file size is divisible by 4 */
+    ssize_t rc;
     int srcFile;
     unsigned char* contents;
-    int count;
+    unsigned count;
     
-    if(file->size % 4 != 0)
-        return BKERROR_WRITE_BOOT_FILE_4;
+    numTrailingBytes = file->size % 4;
     
-    contents = malloc(file->size);
+    contents = malloc(file->size + numTrailingBytes);
     if(contents == NULL)
         return BKERROR_OUT_OF_MEMORY;
     
+    /* make sure the extra bytes i added are 0s */
+    memset(contents + file->size, 0, numTrailingBytes);
+    
     if(file->onImage)
     /* read file from original image */
     {
-        lseek(oldImage, file->offset, SEEK_SET);
+        bkSeekSet(oldImage, file->offset, SEEK_SET);
         
-        rc = read(oldImage, contents, file->size);
-        if(rc != file->size)
+        rc = bkRead(oldImage, contents, file->size);
+        if(rc == -1 || rc != (int)(file->size))
         {
             free(contents);
             return BKERROR_READ_GENERIC;
@@ -432,27 +434,22 @@ int bootInfoTableChecksum(int oldImage, FileToWrite* file, unsigned* checksum)
     else
     /* read file from fs */
     {
-        srcFile = open(file->pathAndName, O_RDONLY);
+        srcFile = open(file->pathAndName, O_RDONLY, 0);
         if(srcFile == -1)
         {
             free(contents);
             return BKERROR_OPEN_READ_FAILED;
         }
         
-        rc = read(srcFile, contents, file->size);
-        if(rc != file->size)
+        rc = bkRead(srcFile, contents, file->size);
+        
+        bkClose(srcFile);
+        
+        if(rc == -1 || rc != (int)(file->size))
         {
-            close(srcFile);
             free(contents);
             return BKERROR_READ_GENERIC;
         }
-        
-        rc = close(srcFile);
-        if(rc < 0)
-        {
-            free(contents);
-            return BKERROR_EXOTIC;
-        }
     }
     
     *checksum = 0;
@@ -602,7 +599,10 @@ int writeByteBlockFromFile(int src, VolInfo* volInfo, unsigned numBytes)
     
     for(count = 0; count < numBlocks; count++)
     {
-        rc = read(src, volInfo->readWriteBuffer, READ_WRITE_BUFFER_SIZE);
+        if(volInfo->stopOperation)
+            return BKERROR_OPER_CANCELED_BY_USER;
+
+        rc = bkRead(src, volInfo->readWriteBuffer, READ_WRITE_BUFFER_SIZE);
         if(rc != READ_WRITE_BUFFER_SIZE)
             return BKERROR_READ_GENERIC;
         rc = wcWrite(volInfo, volInfo->readWriteBuffer, READ_WRITE_BUFFER_SIZE);
@@ -612,7 +612,7 @@ int writeByteBlockFromFile(int src, VolInfo* volInfo, unsigned numBytes)
     
     if(sizeLastBlock > 0)
     {
-        rc = read(src, volInfo->readWriteBuffer, sizeLastBlock);
+        rc = bkRead(src, volInfo->readWriteBuffer, sizeLastBlock);
         if(rc != sizeLastBlock)
                 return BKERROR_READ_GENERIC;
         rc = wcWrite(volInfo, volInfo->readWriteBuffer, sizeLastBlock);
@@ -635,9 +635,9 @@ int writeDir(VolInfo* volInfo, DirToWrite* dir, int parentLbNum,
 {
     int rc;
     
-    off_t startPos;
+    bk_off_t startPos;
     int numUnusedBytes;
-    off_t endPos;
+    bk_off_t endPos;
     
     DirToWrite selfDir; /* will have a different filename */
     DirToWrite parentDir;
@@ -727,7 +727,7 @@ int writeDir(VolInfo* volInfo, DirToWrite* dir, int parentLbNum,
     if(filenameTypes & FNTYPE_JOLIET)
         dir->dataLength2 = wcSeekTell(volInfo) - startPos;
     else
-        dir->base.dataLength = wcSeekTell(volInfo) - startPos;
+        dir->dataLength = wcSeekTell(volInfo) - startPos;
     
     /* write subdirectories */
     child = dir->children;
@@ -738,13 +738,13 @@ int writeDir(VolInfo* volInfo, DirToWrite* dir, int parentLbNum,
             if(filenameTypes & FNTYPE_JOLIET)
             {
                 rc = writeDir(volInfo, DIRTW_PTR(child), dir->extentNumber2, 
-                              dir->dataLength2, dir->base.posixFileMode, recordingTime,
+                              dir->dataLength2, BASETW_PTR(dir)->posixFileMode, recordingTime,
                               filenameTypes, false);
             }
             else
             {
-                rc = writeDir(volInfo, DIRTW_PTR(child), dir->base.extentNumber, 
-                              dir->base.dataLength, dir->base.posixFileMode, recordingTime,
+                rc = writeDir(volInfo, DIRTW_PTR(child), BASETW_PTR(dir)->extentNumber, 
+                              dir->dataLength, BASETW_PTR(dir)->posixFileMode, recordingTime,
                               filenameTypes, false);
             }
             if(rc < 0)
@@ -774,11 +774,11 @@ int writeDir(VolInfo* volInfo, DirToWrite* dir, int parentLbNum,
     }
     else
     {
-        rc = write733(volInfo, dir->base.extentNumber);
+        rc = write733(volInfo, BASETW_PTR(dir)->extentNumber);
         if(rc <= 0)
             return rc;
         
-        rc = write733(volInfo, dir->base.dataLength);
+        rc = write733(volInfo, dir->dataLength);
         if(rc <= 0)
             return rc;
     }
@@ -805,11 +805,11 @@ int writeDir(VolInfo* volInfo, DirToWrite* dir, int parentLbNum,
         }
         else
         {
-            rc = write733(volInfo, dir->base.extentNumber);
+            rc = write733(volInfo, BASETW_PTR(dir)->extentNumber);
             if(rc <= 0)
                 return rc;
             
-            rc = write733(volInfo, dir->base.dataLength);
+            rc = write733(volInfo, dir->dataLength);
             if(rc <= 0)
                 return rc;
         }
@@ -853,7 +853,7 @@ int writeDir(VolInfo* volInfo, DirToWrite* dir, int parentLbNum,
                 if(rc <= 0)
                     return rc;
                 
-                rc = write733(volInfo, child->dataLength);
+                rc = write733(volInfo, DIRTW_PTR(child)->dataLength);
                 if(rc <= 0)
                     return rc;
             }
@@ -868,7 +868,7 @@ int writeDir(VolInfo* volInfo, DirToWrite* dir, int parentLbNum,
     if(filenameTypes & FNTYPE_JOLIET)
         return dir->dataLength2;
     else
-        return dir->base.dataLength;
+        return dir->dataLength;
 }
 
 /******************************************************************************
@@ -884,8 +884,8 @@ int writeDr(VolInfo* volInfo, BaseToWrite* node, time_t recordingTime, bool isAD
     unsigned char byte;
     char aString[256];
     unsigned short aShort;
-    off_t startPos;
-    off_t endPos;
+    bk_off_t startPos;
+    bk_off_t endPos;
     unsigned char lenFileId;
     unsigned char recordLen;
     
@@ -1062,7 +1062,7 @@ int writeDr(VolInfo* volInfo, BaseToWrite* node, time_t recordingTime, bool isAD
         
         if(!isSelfOrParent)
         {
-            if(wcSeekTell(volInfo) - startPos < strlen(node->nameRock) + 5)
+            if(wcSeekTell(volInfo) - startPos < (int)strlen(node->nameRock) + 5)
             /* have no room for the NM entry in this directory record */
             {
                 node->offsetForCE = wcSeekTell(volInfo);
@@ -1130,12 +1130,12 @@ int writeDr(VolInfo* volInfo, BaseToWrite* node, time_t recordingTime, bool isAD
 * be written (7.3.1).
 * */
 int writeElToritoBootCatalog(VolInfo* volInfo, 
-                             off_t* bootRecordSectorNumberOffset)
+                             bk_off_t* bootRecordSectorNumberOffset)
 {
     unsigned char buffer[NBYTES_LOGICAL_BLOCK];
     int rc;
     
-    bzero(buffer, NBYTES_LOGICAL_BLOCK);
+    memset(buffer, 0, NBYTES_LOGICAL_BLOCK);
     
     if(wcSeekTell(volInfo) % NBYTES_LOGICAL_BLOCK != 0)
     /* file pointer not at sector boundary */
@@ -1196,12 +1196,12 @@ int writeElToritoBootCatalog(VolInfo* volInfo,
 * Returns the offset where the boot catalog sector number should 
 * be written (7.3.1).
 * */
-int writeElToritoVd(VolInfo* volInfo, off_t* bootCatalogSectorNumberOffset)
+int writeElToritoVd(VolInfo* volInfo, bk_off_t* bootCatalogSectorNumberOffset)
 {
     char buffer[NBYTES_LOGICAL_BLOCK];
     int rc;
     
-    bzero(buffer, NBYTES_LOGICAL_BLOCK);
+    memset(buffer, 0, NBYTES_LOGICAL_BLOCK);
     
     if(wcSeekTell(volInfo) % NBYTES_LOGICAL_BLOCK != 0)
     /* file pointer not at sector boundary */
@@ -1219,7 +1219,7 @@ int writeElToritoVd(VolInfo* volInfo, off_t* bootCatalogSectorNumberOffset)
     /* unused 32 bytes, must be 0 (bzero at start took care of this) */
     /* boot catalog location, 4 byte intel format. written later. */
     *bootCatalogSectorNumberOffset = wcSeekTell(volInfo) + 71;
-    //write731ToByteArray(&(buffer[71]), bootCatalogSectorNumber);
+    /*write731ToByteArray(&(buffer[71]), bootCatalogSectorNumber);*/
     /* the rest of this sector is unused, must be set to 0 */
     /* END SETUP BOOT record volume descriptor sector */
     
@@ -1234,6 +1234,7 @@ int writeElToritoVd(VolInfo* volInfo, off_t* bootCatalogSectorNumberOffset)
 * writeFileContents()
 * Write file contents into an extent and also write the file's location and 
 * size into the directory records back in the tree.
+* Also write location and size for symbolic links.
 * */
 int writeFileContents(VolInfo* volInfo, DirToWrite* dir, int filenameTypes)
 {
@@ -1242,7 +1243,7 @@ int writeFileContents(VolInfo* volInfo, DirToWrite* dir, int filenameTypes)
     BaseToWrite* child;
     int numUnusedBytes;
     int srcFile;
-    off_t endPos;
+    bk_off_t endPos;
     
     child = dir->children;
     while(child != NULL)
@@ -1260,28 +1261,26 @@ int writeFileContents(VolInfo* volInfo, DirToWrite* dir, int filenameTypes)
             
             child->extentNumber = wcSeekTell(volInfo) / NBYTES_LOGICAL_BLOCK;
             if(volInfo->scanForDuplicateFiles)
-            {//printf("file '%s': ", child->nameRock);
+            {
                 if(FILETW_PTR(child)->location->extentNumberWrittenTo == 0)
                 /* file not yet written */
-                {//printf("writing to 0x%X\n", child->extentNumber * NBYTES_LOGICAL_BLOCK);
+                {
                     FILETW_PTR(child)->location->extentNumberWrittenTo = child->extentNumber;
                 }
                 else
-                {//printf("not writing, already at 0x%X\n", FILETW_PTR(child)->location->extentNumberWrittenTo * NBYTES_LOGICAL_BLOCK);
+                {
                     child->extentNumber = FILETW_PTR(child)->location->extentNumberWrittenTo;
                     needToCopy = false;
                 }
             }
             
-            child->dataLength = FILETW_PTR(child)->size;
-            
             if(volInfo->bootMediaType != BOOT_MEDIA_NONE && 
                volInfo->bootRecordIsVisible &&
                FILETW_PTR(child)->origFile == volInfo->bootRecordOnImage)
             /* this file is the boot record. write its sector number in 
             * the boot catalog */
             {
-                off_t currPos;
+                bk_off_t currPos;
                 
                 currPos = wcSeekTell(volInfo);
                 
@@ -1298,32 +1297,42 @@ int writeFileContents(VolInfo* volInfo, DirToWrite* dir, int filenameTypes)
                 if(FILETW_PTR(child)->onImage)
                 /* copy file from original image to new one */
                 {
-                    lseek(volInfo->imageForReading, FILETW_PTR(child)->offset, 
-                          SEEK_SET);
+                    readSeekSet(volInfo, FILETW_PTR(child)->offset, 
+                                SEEK_SET);
                     
                     rc = writeByteBlockFromFile(volInfo->imageForReading, 
-                                             volInfo, FILETW_PTR(child)->size);
+                                                volInfo, FILETW_PTR(child)->size);
                     if(rc < 0)
                         return rc;
                 }
                 else
                 /* copy file from fs to new image */
                 {
-                    srcFile = open(FILETW_PTR(child)->pathAndName, O_RDONLY);
+                    /* UPDATE the file's size, in case it's changed since we added it */
+                    BkStatStruct statStruct;
+                    
+                    rc = bkStat(FILETW_PTR(child)->pathAndName, &statStruct);
+                    if(rc != 0)
+                        return BKERROR_STAT_FAILED;
+                    
+                    if(statStruct.st_size > 0xFFFFFFFF)
+                    /* size won't fit in a 32bit variable on the iso */
+                        return BKERROR_EDITED_WRITE_TOO_BIG;
+                    
+                    FILETW_PTR(child)->size = statStruct.st_size;
+                    /* UPDATE the file's size, in case it's changed since we added it */
+                    
+                    srcFile = open(FILETW_PTR(child)->pathAndName, O_RDONLY, 0);
                     if(srcFile == -1)
                         return BKERROR_OPEN_READ_FAILED;
                     
                     rc = writeByteBlockFromFile(srcFile, 
-                                             volInfo, FILETW_PTR(child)->size);
-                    if(rc < 0)
-                    {
-                        close(srcFile);
-                        return rc;
-                    }
+                                                volInfo, FILETW_PTR(child)->size);
+                    
+                    bkClose(srcFile);
                     
-                    rc = close(srcFile);
                     if(rc < 0)
-                        return BKERROR_EXOTIC;
+                        return rc;
                 }
                 
                 /* fill extent with zeroes */
@@ -1336,16 +1345,18 @@ int writeFileContents(VolInfo* volInfo, DirToWrite* dir, int filenameTypes)
             
             endPos = wcSeekTell(volInfo);
             
-            if(volInfo->bootMediaType != BOOT_MEDIA_NONE && 
-               volInfo->bootRecordIsVisible &&
-               FILETW_PTR(child)->origFile == volInfo->bootRecordOnImage)
-            /* this file is the boot record. assume it's isolinux and write the 
-            * boot info table */
+            bool isIsolinux;
+            rc = wroteIsolinuxBootRecord(volInfo, FILETW_PTR(child), &isIsolinux);
+            if(rc < 0)
+                return rc;
+            
+            if(isIsolinux)
+            /* write the boot info table for the isolinux boot record */
             {
                 unsigned char bootInfoTable[56];
                 unsigned checksum;
                 
-                bzero(bootInfoTable, 56);
+                memset(bootInfoTable, 0, 56);
                 
                 /* go to the offset in the file where the boot info table is */
                 wcSeekSet(volInfo, child->extentNumber * 
@@ -1377,7 +1388,7 @@ int writeFileContents(VolInfo* volInfo, DirToWrite* dir, int filenameTypes)
             if(rc <= 0)
                 return rc;
             
-            rc = write733(volInfo, child->dataLength);
+            rc = write733(volInfo, FILETW_PTR(child)->size);
             if(rc <= 0)
                 return rc;
             
@@ -1390,7 +1401,7 @@ int writeFileContents(VolInfo* volInfo, DirToWrite* dir, int filenameTypes)
                 if(rc <= 0)
                     return rc;
                 
-                rc = write733(volInfo, child->dataLength);
+                rc = write733(volInfo, FILETW_PTR(child)->size);
                 if(rc <= 0)
                     return rc;
             }
@@ -1404,6 +1415,38 @@ int writeFileContents(VolInfo* volInfo, DirToWrite* dir, int filenameTypes)
             if(rc < 0)
                 return rc;
         }
+        else if( IS_SYMLINK(child->posixFileMode) )
+        {
+            /* WRITE symlink location and size (0) */
+            endPos = wcSeekTell(volInfo);
+            
+            wcSeekSet(volInfo, child->extentLocationOffset);
+            
+            rc = write733(volInfo, 0);
+            if(rc <= 0)
+                return rc;
+            
+            rc = write733(volInfo, 0);
+            if(rc <= 0)
+                return rc;
+            
+            if(filenameTypes & FNTYPE_JOLIET)
+            /* also update location and size on joliet tree */
+            {
+                wcSeekSet(volInfo, child->extentLocationOffset2);
+                
+                rc = write733(volInfo, 0);
+                if(rc <= 0)
+                    return rc;
+                
+                rc = write733(volInfo, 0);
+                if(rc <= 0)
+                    return rc;
+            }
+            
+            wcSeekSet(volInfo, endPos);
+            /* END WRITE symlink location and size (0)  */
+        }
         
         child = child->next;
         
@@ -1413,12 +1456,12 @@ int writeFileContents(VolInfo* volInfo, DirToWrite* dir, int filenameTypes)
 }
 
 /* field size must be even. !!check all calls to make sure */
-int writeJolietStringField(VolInfo* volInfo, const char* name, int fieldSize)
+int writeJolietStringField(VolInfo* volInfo, const char* name, size_t fieldSize)
 {
     char jolietName[512]; /* don't see why would ever want 
                           * to write a longer one */
     int srcCount;
-    int destCount;
+    size_t destCount;
     int rc;
     
     srcCount = 0;
@@ -1453,12 +1496,12 @@ int writeJolietStringField(VolInfo* volInfo, const char* name, int fieldSize)
 /* write NM that won't fit in a directory record */
 int writeLongNM(VolInfo* volInfo, BaseToWrite* node)
 {
-    off_t startPos;
-    int fullNameLen;
+    bk_off_t startPos;
+    size_t fullNameLen;
     unsigned char CErecord[28];
     bool fitsInOneNM;
-    int firstNMlen;
-    off_t endPos;
+    size_t firstNMlen;
+    bk_off_t endPos;
     int rc;
     int lenOfCE;
     
@@ -1568,7 +1611,7 @@ int writePathTable(VolInfo* volInfo, const DirToWrite* tree, bool isTypeL,
     int level;
     int* dirsPerLevel; /* a dynamic array of the number of dirs per level */
     int numDirsSoFar;
-    off_t origPos;
+    bk_off_t origPos;
     int numBytesWritten;
     int rc;
     
@@ -1811,7 +1854,7 @@ int writeRockER(VolInfo* volInfo)
     return 1;
 }
 
-int writeRockNM(VolInfo* volInfo, char* name, int nameLen, bool doesContinue)
+int writeRockNM(VolInfo* volInfo, char* name, size_t nameLen, bool doesContinue)
 {
     int rc;
     char recordStart[5];
@@ -1849,7 +1892,6 @@ int writeRockPX(VolInfo* volInfo, unsigned posixFileMode, bool isADir)
 {
     int rc;
     unsigned char record[36];
-    //DirToWriteLL* nextDir;
     unsigned posixFileLinks;
     
     /* identification */
@@ -1866,11 +1908,13 @@ int writeRockPX(VolInfo* volInfo, unsigned posixFileMode, bool isADir)
     write733ToByteArray(&(record[4]), posixFileMode);
     
     /* POSIX file links */
-    // this i think is number of subdirectories + 2 (self and parent)
-    // and 1 for a file
-    // it's probably not used on read-only filesystems
-    // to add it, i would need to pass the number of links in a parent dir
-    // recursively in writeDir(). brrrrr.
+    /*
+    * this i think is number of subdirectories + 2 (self and parent)
+    * and 1 for a file
+    * it's probably not used on read-only filesystems
+    * to add it, i would need to pass the number of links in a parent dir
+    * recursively in writeDir(). brrrrr.
+    */
     if(isADir)
         posixFileLinks = 2;
     else
@@ -1880,7 +1924,7 @@ int writeRockPX(VolInfo* volInfo, unsigned posixFileMode, bool isADir)
     /* END POSIX file links */
     
     /* posix file user id, posix file group id */
-    bzero(&(record[20]), 16);
+    memset(&(record[20]), 0, 16);
     
     rc = wcWrite(volInfo, (char*)record, 36);
     if(rc <= 0)
@@ -1891,12 +1935,12 @@ int writeRockPX(VolInfo* volInfo, unsigned posixFileMode, bool isADir)
 
 int writeRockSL(VolInfo* volInfo, SymLinkToWrite* symlink, bool doWrite)
 {
-    int stringCount;
-    int targetLen;
-    int numBytesNeeded;
-    int numBytesToSkip;
+    size_t stringCount;
+    size_t targetLen;
+    size_t numBytesNeeded;
+    size_t numBytesToSkip;
     unsigned char* record;
-    int recordCount;
+    size_t recordCount;
     int rc;
     
     targetLen = strlen(symlink->target);
@@ -1907,7 +1951,6 @@ int writeRockSL(VolInfo* volInfo, SymLinkToWrite* symlink, bool doWrite)
     stringCount = 0;
     while(stringCount < targetLen)
     {
-        int numBytesToSkip;
         char* nextSlash;
         
         if(symlink->target[stringCount] == '/')
@@ -1948,7 +1991,7 @@ int writeRockSL(VolInfo* volInfo, SymLinkToWrite* symlink, bool doWrite)
     }
     
     if(!doWrite)
-        return 5 + numBytesNeeded;
+        return (int)(5 + numBytesNeeded);
     
     if(numBytesNeeded > NCHARS_SYMLINK_TARGET_MAX - 1)
         return BKERROR_SYMLINK_TARGET_TOO_LONG;
@@ -1969,7 +2012,6 @@ int writeRockSL(VolInfo* volInfo, SymLinkToWrite* symlink, bool doWrite)
     recordCount = 5;
     while(stringCount < targetLen)
     {
-        int numBytesToSkip;
         char* nextSlash;
         
         if(symlink->target[stringCount] == '/')
@@ -2034,7 +2076,7 @@ int writeRockSL(VolInfo* volInfo, SymLinkToWrite* symlink, bool doWrite)
     
     free(record);
 
-    return 5 + numBytesNeeded;
+    return (int)(5 + numBytesNeeded);
 }
 
 /* This doesn't need support for CE because it's only written in one place,
@@ -2105,19 +2147,19 @@ int writeVdsetTerminator(VolInfo* volInfo)
 * -rootdr location, size are in bytes
 * -note strings are not terminated on image
 */
-int writeVolDescriptor(VolInfo* volInfo, off_t rootDrLocation,
-                       unsigned rootDrSize, off_t lPathTableLoc, 
-                       off_t mPathTableLoc, unsigned pathTableSize, 
+int writeVolDescriptor(VolInfo* volInfo, bk_off_t rootDrLocation,
+                       unsigned rootDrSize, bk_off_t lPathTableLoc, 
+                       bk_off_t mPathTableLoc, unsigned pathTableSize, 
                        time_t creationTime, bool isPrimary)
 {
     int rc;
-    int count;
+    size_t count;
     
     unsigned char byte;
     unsigned char aString[129];
     unsigned anUnsigned;
     unsigned short anUnsignedShort;
-    size_t currPos;
+    bk_off_t currPos;
     
     /* VOLUME descriptor type */
     if(isPrimary)
@@ -2191,13 +2233,13 @@ int writeVolDescriptor(VolInfo* volInfo, off_t rootDrLocation,
     
     /* VOLUME space size (number of logical blocks, absolutely everything) */
     /* it's safe to not use wcSeek() here since everything is left as it is */
-    currPos = lseek(volInfo->imageForWriting, 0, SEEK_CUR);
+    currPos = bkSeekTell(volInfo->imageForWriting);
     
-    lseek(volInfo->imageForWriting, 0, SEEK_END);
-    anUnsigned = lseek(volInfo->imageForWriting, 0, SEEK_CUR) / 
+    bkSeekSet(volInfo->imageForWriting, 0, SEEK_END);
+    anUnsigned = bkSeekTell(volInfo->imageForWriting) / 
                  NBYTES_LOGICAL_BLOCK;
     
-    lseek(volInfo->imageForWriting, currPos, SEEK_SET);
+    bkSeekSet(volInfo->imageForWriting, currPos, SEEK_SET);
     
     rc = write733(volInfo, anUnsigned);
     if(rc <= 0)
@@ -2465,3 +2507,41 @@ int writeVolDescriptor(VolInfo* volInfo, off_t rootDrLocation,
     
     return 1;
 }
+
+/******************************************************************************
+* wroteIsolinuxBootRecord()
+* Check whether the file already written to the new iso was a boot record.
+* */
+int wroteIsolinuxBootRecord(VolInfo* volInfo, FileToWrite* file, 
+                            bool* isIsolinux)
+{
+    *isIsolinux = false;
+    
+    if(volInfo->bootMediaType == BOOT_MEDIA_NO_EMULATION && 
+       volInfo->bootRecordIsVisible &&
+       file->origFile == volInfo->bootRecordOnImage)
+    /* Likely true, do one more check to make sure. The extra check
+    * is needed for Windows XP isos with SP2 added by c't slipstreamer */
+    {
+        bk_off_t origPos;
+        int rc;
+        char fourBytes[4];
+        
+        origPos = wcSeekTell(volInfo);
+        
+        wcSeekSet(volInfo, BASETW_PTR(file)->extentNumber * 
+                  NBYTES_LOGICAL_BLOCK + 8);
+        
+        rc = bkRead(volInfo->imageForWriting, fourBytes, 4);
+        if(rc != 4)
+            return BKERROR_READ_GENERIC;
+        
+        if(fourBytes[0] == 16 && fourBytes[1] == 0 && 
+           fourBytes[2] == 0 && fourBytes[3] == 0)
+            *isIsolinux = true;
+        
+        wcSeekSet(volInfo, origPos);
+    }
+    
+    return 1;
+}
